{===============================================================================
 Paradox database upgrader (PXUPGR.DLL) import unit.

         Roman Krejci-RKS,
         Pujmanove 1552,
  140 00 Prague 4,
         Czech Republic

 http://www.rksolution.cz,
 EMail INFO@rksolution.cz
 ===============================================================================}

unit PxUpgrH;


interface
uses Windows,Sysutils,BDE,DB,DBTABLES;

Const
{$I PXUPGRH.INC} { contains string constants to be translated }

{****************************************************************************************}
{ PXUPGRLICENSEPASSWORD - password that, passed to PXUPGR.DLL via pxupLicPassword call   }
{ (see below), makes PXUPGR.DLL registered                                               }
{****************************************************************************************}
  PXUPGRLICENSEPASSWORD = '';       { License password is available for registered users }

  {  Error codes }

  pxuErr_AlreadyUpdated   = -1; { Duplicete update action }
  pxuErr_None             = 0;  { OK }

  {A. command parse time errors }
  pxuErr_MissingOperand            = 1;  { missing operand }
  pxuErr_UnknownOperation          = 2;  { unknown operation }
  pxuErr_ArityError                = 3;  { incorrect number of operands }
  pxuErr_TypeMismatch              = 4;  { type mismatch }
  pxuErr_MissingLeftPar            = 5;  { "(" expected }
  pxuErr_MissingRightPar           = 6;  { ")" expected }
  pxuErr_UnterminatedString        = 7;  { unterminated string }
  pxuErr_TooComplicated            = 8;  { too complicated }
  pxuErr_NotACommand               = 9;  { Expression does not result in database update operation }
  pxuErr_DriverNotSupported        = 10; { Driver not supported }
  pxuErr_NotSupportedByDriver      = 11; { Command not supported by selected driver type }

  {B. command execution time errors }
  pxuErr_UnknownError              = 100; { none of the below }
  pxuErr_InvalidOpType             = 101; { Invalid operand type }
  pxuErr_OverUnderFlow             = 102; { numeric Over/Underflow }
  pxuErr_DivZero                   = 103; { division by zero }
  pxuErr_InvalidOperand            = 104; { invalid operand value (i.e. SQRT(-1) }
  pxuErr_OutOfMemory               = 105; { out of memory }
  pxuErr_UnknowType                = 106; { unknown/unsupported type in expression }

  pxuErr_NotParadoxTable           = 201; { Command operates on non-Paradox table in Paradox database }
  pxuErr_NotDBaseTable             = 202; { Command operates on non-DBase table in DBase database }
  pxuErr_NotAsciiTable             = 203; { Command operates on non-Ascii table in ASCII database }
  pxuErr_cannotLoadIDAPI           = 204; { BDE not installed or interface mismatch }
  pxuErr_HigherBDEVersionNeeded    = 205; { Idapi32.dll must be at least 3.5.0.21 }

  { Error codes above pxuErr_BDEBase inclusive are BDE error codes }
  pxuErr_BDEBase                   = ERRBASE_SYSTEM;

  { DLL client error }
  pxuErr_DLLNotLoaded              = 999; { operation failed because PXUPGR.DLL was not loaded }

  pxuErr_User                      = 1000;{ base for application defined errors }




Type

{--------------------------------------------------------------------------------------
 Callback for unknown password retrieval. Address of the function can be registered
  by calling pxupRegisterPswdCallback (see below) from within the main application.

  Arguments
      szTablename (in)... pointer to zero terminated string holding name of the password
                          protected table.
      Returns         ... function should return address of zero terminated string
                          holding the table's master password. If the user cannot
                          provide the password, function should return NIL. In this case
                          BDE error code DBIERR_NOTSUFFTABLERIGHTS will be returned
                          by currently executed PXUPGR command that accessed
                          the protected table.

      Comment: PXUPGR.DLL verifies the password before the process proceeds. If the password
               turns to be incorrect, callback function can be called repeatedly.
--------------------------------------------------------------------------------------}
  TUnknownPasswordCallback = function(szTablename: pChar): pChar; stdcall;

{----------------------------------}
{ Declaration of imported routines }
{----------------------------------}
{ pxupExecLine
  ------------
  Applies the command szCommand to the database szDbsename. Returns error code.
  The function first parses the command. If the command cannot be parsed, function
  exits and parse time error code is returned, otherwise the routine tries to open
  the given database and apply the command. }   
  TpxupExecLine    = Function (szDbsename,szDriverType,szCommand: pChar): integer; stdcall;

{ pxupCheckSyntax
  ---------------
  Checks whether szCommand is correct Pxupgr command. Returns pxuErr_None,
  or corresponding parse time error code. }
  TpxupCheckSyntax = Function (szDriverType,szCommand: pChar): integer; stdcall;


{ pxupAddPassword
  ---------------
  Fetches one master password into DLL so that there is no need to invoke callback
  for unknown password retrieval (see pxupRegisterPswdCallback) during PXUPGR
  command execution. pxupAddPassword should be called and all passwords should
  be provided (by possibly multiple pxuAddPassword calls) before pxupExecLine
  is called for the first time.}
  TpxupAddPassword = Procedure(szPswd: pChar); stdcall;

{ pxupRegisterPswdCallback
  ------------------------
  (Un)Registers callback routine for unknown password retrieval.
  ClbckAddr should point to the function having TUnknownPasswordCallback (see above)
  type. Callback function can be unregistered by passing NIL as ClbckAddr argument.
  Returns address of previously registered callback.
  Callback is invoked if password protected table for which the password was not provided
  by pxupAddPassword call must be opened during database upgrade operation. }
  TpxupRegisterPswdCallback = Function(ClbckAddr: pointer): pointer; stdcall;

{ pxupLicPassword
  ------------------------
  Provides license string. If szLicensePassword is correct password,
  DLL becomes registered. You should call pxupLicPassword immediately
  after the DLL has been loaded. You will receive DLL password after
  registration }
  TpxupLicPassword= procedure(szLicensePassword: pChar); stdcall;

Var
{ DLL imported routines. Addresses are set within "pxupgrInit" function. }
  pxupExecLine             :  TpxupExecLine;
  pxupCheckSyntax          :  TpxupCheckSyntax;
  pxupAddPassword          :  TpxupAddPassword;
  pxupRegisterPswdCallback :  TpxupRegisterPswdCallback;
  pxupLicPassword          :  TpxupLicPassword;


{----------------------------------------------}
{ Additional routines implemented in this unit }
{----------------------------------------------}
{ GetPXUPErrorMsg
  ---------------
  Returns error description for given error code }
Function GetPXUPErrorMsg(ErrCode: Integer): string;

{ pxupgrInit
  ----------
  Loads pxupgr.dll located in the specified directory and sets the addresses of
  the imported routines. If DllDir=empty string, Windows default search path is used.
  Returns pxuErr_None if successful, otherwise returns either pxuErr_DLLNotLoaded 
  (if the DLL cannot be found), or pxuErr_cannotLoadIDAPI (if there is no BDE installed).
  Routine keeps track of the number of times the DLL was loaded/unloaded (by calling
  pxupgrInit/pxupgrDone). LoadLibrary takes place only if this counter is zero,
  therefore each pxupgrInit call must be matched by exactly one pxupgrDone call }
function pxupgrInit(DllDir: string): integer;

{ pxupgrDone
  ----------
  Routine keeps track of the number of times the DLL was loaded/unloaded by calling
  pxupgrInit/pxupgrDone. DLL is unloaded and addresses of imported routines are
  set to NIL if the counter reaches zero }
procedure pxupgrDone;

implementation

Function GetPXUPErrorMsg(ErrCode: Integer): string;
begin
  if ErrCode >= pxuErr_BDEBase then begin
    SetLength(Result,DBIMAXMSGLEN);
    DbiGetErrorString(ErrCode,pChar(Result));
    SetLength(Result,StrLen(pChar(Result)));
  end else case ErrCode of
  pxuErr_AlreadyUpdated            : Result := pxuMsg_Err_AlreadyUpdated;
  pxuErr_None                      : Result := pxuMsg_Err_None;
  {A. script line parse errors }
  pxuErr_MissingOperand            : Result := pxuMsg_Err_MissingOperand;
  pxuErr_UnknownOperation          : Result := pxuMsg_Err_UnknownOperation;
  pxuErr_ArityError                : Result := pxuMsg_Err_ArityError;
  pxuErr_TypeMismatch              : Result := pxuMsg_Err_TypeMismatch;
  pxuErr_MissingLeftPar            : Result := pxuMsg_Err_MissingLeftPar;
  pxuErr_MissingRightPar           : Result := pxuMsg_Err_MissingRightPar;
  pxuErr_UnterminatedString        : Result := pxuMsg_Err_UnterminatedString;
  pxuErr_TooComplicated            : Result := pxuMsg_Err_TooComplicated;
  pxuErr_NotACommand               : Result := pxuMsg_Err_NotACommand;
  pxuErr_DriverNotSupported        : Result := pxuMsg_Err_DriverNotSupported;
  pxuErr_NotSupportedByDriver      : Result := pxuMsg_Err_NotSupportedByDriver;
  {B. script line execution errors }
  pxuErr_InvalidOpType             : Result := pxuMsg_Err_InvalidOpType;
  pxuErr_OverUnderFlow             : Result := pxuMsg_Err_OverUnderFlow;
  pxuErr_DivZero                   : Result := pxuMsg_Err_DivZero;
  pxuErr_InvalidOperand            : Result := pxuMsg_Err_InvalidOperand;
  pxuErr_OutOfMemory               : Result := pxuMsg_Err_OutOfMemory;
  pxuErr_UnknowType                : Result := pxuMsg_Err_UnknowType;
  pxuErr_NotParadoxTable           : Result := pxuMsg_Err_NotParadoxTable;
  pxuErr_NotDBaseTable             : Result := pxuMsg_Err_NotDBaseTable;
  pxuErr_NotAsciiTable             : Result := pxuMsg_Err_NotAsciiTable;
  pxuErr_CannotLoadIDAPI           : Result := pxuMsg_Err_CannotLoadIDAPI;
  pxuErr_HigherBDEVersionNeeded    : Result := pxuMsg_Err_HigherBDEVersionNeeded;
  pxuErr_DLLNotLoaded              : Result := pxuMsg_Err_DLLNotLoaded;
  else Result := Format(pxuMsg_Err_UnknownError,[ErrCode]);
  end;
  if ErrCode >= pxuErr_UnknownError {execution time errors } then
    Result := Format(pxuMsg_ExecTimeError,[Result])
  else
  if ErrCode > 0 { parseTimeErrors } then
    Result := Format(pxuMsg_ParseTimeError,[Result]);
end;


Const DLLname = 'pxupgr.dll';

Var
  CounterLoaded: integer;
  DLLHandle    : tHandle;

procedure pxupgrDone;
begin
  if DLLHandle <> 0 then begin
    Dec(CounterLoaded);
    if CounterLoaded <= 0 then begin
      FreeLibrary(DLLHandle);
      DLLHandle := 0;
      CounterLoaded := 0;
      pxupExecLine := NIL;
      pxupCheckSyntax := NIL;
      pxupAddPassword := NIL;
      pxupRegisterPswdCallback := NIL;
    end;
  end;
end;

function pxupgrInit(DllDir: string): integer;
begin
  Result := pxuErr_DLLNotLoaded; // assume failure
  if DLLHandle = 0 then begin
    CounterLoaded := 0;
  { Main application should initialize default BDE session before loading the DLL: }
    if not (IsLibrary or session.Active) then  Try
      session.Active := true;
    except
      Result := pxuErr_cannotLoadIDAPI;
    end;
    if Result = pxuErr_DLLNotLoaded then begin
      if (length(DllDir)>0) and not (DllDir[length(DllDir)] in [#0,':','\']) then DllDir := DllDir + '\';
      DLLHandle := LoadLibrary(pChar(DllDir+DLLName));
      if DLLHandle <> 0 then begin
        pxupExecLine             := GetprocAddress(DLLHandle, lpcstr(longint(1))); // index 1
        pxupCheckSyntax          := GetprocAddress(DLLHandle, lpcstr(longint(2))); // index 2
        pxupAddPassword          := GetprocAddress(DLLHandle, lpcstr(longint(3))); // index 3
        pxupRegisterPswdCallback := GetprocAddress(DLLHandle, lpcstr(longint(4))); // index 4
        pxupLicPassword          := GetprocAddress(DLLHandle, lpcstr(longint(5))); // index 5
        if not (assigned(pxupAddPassword) and
                assigned(pxupExecLine) and
                assigned(pxupCheckSyntax) and
                assigned(pxupRegisterPswdCallback) and
                assigned(pxupLicPassword))
        then begin
          pxupgrDone;
        end else begin
          pxupLicPassword(PXUPGRLICENSEPASSWORD);
        end;
      end;
    end;
  end;
  if DLLHandle <> 0 then begin
    inc(CounterLoaded);
    Result := pxuErr_None;
  end;
end;


end.
