{===============================================================================
 "Paradox database version upgrader" demo application.
        Roman Krejci-RKS,
        Pujmanove 1552,
 140 00 Prague 4,
        Czech Republic
 http://www.rksolution.cz, EMail INFO@rksolution.cz


 MAIN form unit
 ===============================================================================}

unit Main;

interface

uses
  Windows, BDE, SysUtils, Messages, Classes, Graphics, Controls, Dialogs, FileCtrl,
  StdCtrls, Forms, DBCtrls, DB, DBGrids, DBTables, Grids, Mask, ExtCtrls,
  pxUpgrH,  //  PXUPGR.DLL import unit
  RunScri,  //  implements RunPXUPScript function
  UpgrInfo; //  splash form with database upgrade progress gauge

type
  TfrmMain = class(TForm)
    DBGrid1: TDBGrid;
    DBNavigator: TDBNavigator;
    Panel1: TPanel;
    DataSource1: TDataSource;
    Panel3: TPanel;
    Table1: TTable;
    ixCombo: TComboBox;
    LabelIxCombo: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure Table1AfterOpen(DataSet: TDataSet);
    procedure ixComboChange(Sender: TObject);
    procedure Table1AfterClose(DataSet: TDataSet);
  private
    { private declarations }
    frmUpgrInfo: TfrmUpgrInfo;
    function ConnectToDatabase(const Dbsename: string): boolean;
    procedure UpdateIndexCombo(Tbl: tTable);
  public
    { public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.DFM}

const
{Startup message:}
  sStartupMessage = ' Upon the first launch, this application creates database tables in'#13+
                    ' EXE subfolder named DATA. The structure of the database created'#13+
                    ' is determined by the DBVer.txt script file. You can add your own '#13+
                    ' commands to this script file in order to change the database structure';

  ScriptFileName = 'dbver.txt';  // script file - located in EXE directory
  LogFileName    = 'dbupgr.log'; // database version upgrade log file - located in database folder

{ callback function for database version upgrade process }
function PXUPGRCallback(pRec: pPXUPCallbackRecord): boolean;
begin
  Result := True;
  if Frmmain.frmUpgrInfo = nil then begin
    Frmmain.frmUpgrInfo := TfrmUpgrInfo.Create(Frmmain);
    Frmmain.frmUpgrInfo.Show;
  end else begin
    Frmmain.frmUpgrInfo.Gauge1.Progress := Round(100*pRec.LinesDone/pRec.LinesApplicable);
    Result := NOT Frmmain.frmUpgrInfo.Aborted;
  end;
  Application.processmessages;
end;

{ Returns folder for database Dbsename. Result includes terminating backslash. }
Function GetDatabaseFolder(const Dbsename: string): string;
var List: tStringList;
begin
  if Session.IsAlias(Dbsename) then begin     { Find physical path for alias }
    List := tStringList.Create;
    try
      session.GetAliasParams(DBseName,list);
      Result := list.values['PATH'];
    finally
      list.free;
    end;
  end else
    Result := Dbsename; { Dbsename is already database path }
 { Pad result with backslash }
  if (length(Result)>0) and not (Result[length(Result)] in [#0,':','\']) then Result := Result + '\';
end;

{ Connects to the database - this has been made separate function.
  Returns true if pxupgr script was successfuly processed, false otherwise }
function TFrmmain.ConnectToDatabase(const Dbsename: string): boolean;
var UpgrResult: Integer;
begin
  Try
    UpgrResult := RunPXUPScript(
                      ExtractFilepath(Paramstr(0))+'..',           // DLL location - adjust when appropriate
                      Dbsename,                                    // Database name : func.argument
                      GetDatabaseFolder(Dbsename) + LogFilename,   // Log file : database folder + hardwired constant
                      szParadox,                                   // Driver type
                      NIL,                                         // known database passwords: none
                      ExtractFilepath(Paramstr(0))+ScriptFileName, // Sript file : application folder + hardwired constant
                      PXUPGRCallback);                             // Process callback - implemented in this unit
    Result := UpgrResult <= 0;
  Finally
    frmUpgrInfo.Free;
    frmUpgrInfo := nil;
  End;
  if not Result then begin
    if UpgrResult = AppErr_UserAbort then
      showmessage(AppMsg_Err_UserAbort)
    else
      showmessage(Format('Error in database upgrade - %s',[GetPXUPErrorMsg(UpgrResult)]))
  end else begin { open tables }
    Table1.databasename := Dbsename;
    Table1.tablename := 'Country.db';
    Table1.Open;
  end;
end;

{ Updates ixCombo.Items :
    strings = Index field names
    objects = IndexDef.Items }
procedure TfrmMain.UpdateIndexCombo(Tbl: tTable);
var i: integer;
begin
  ixCombo.Items.Clear;
  if (Tbl <> nil) and Tbl.Active then begin
    Tbl.IndexDefs.Update;
    with Tbl.IndexDefs do
      For i := 1 to Count do
        if ixDescending in Items[i-1].Options then
          ixCombo.Items.AddObject(Items[i-1].Fields+'   (desc.)',Items[i-1])
        else
          ixCombo.Items.AddObject(Items[i-1].Fields,Items[i-1]);
  end;
  ixCombo.Visible := ixCombo.Items.Count > 0;
  LabelIxCombo.Visible := ixCombo.Visible;
  if ixCombo.Visible then ixCombo.ItemIndex := 0;
end;

{ ------- EVENT HANDLERS --------- }

procedure TfrmMain.FormCreate(Sender: TObject);
var Dbsename: string;
begin
  UpdateIndexCombo(nil);
  Dbsename := ExtractFilepath(Paramstr(0))+'data';
  if not DirectoryExists(Dbsename) then begin
    if MessageDlg(sStartupMessage,mtInformation,[mbOK,mbAbort],0) = mrOK then
      ForceDirectories(Dbsename)
    else
      Dbsename := '';
  end;
{ Connect to the database }
  if not ((Dbsename <> '') and ConnectToDatabase( Dbsename))
   then Application.Terminate;
end;


procedure TfrmMain.Table1AfterOpen(DataSet: TDataSet);
begin
  UpdateIndexCombo(dataset as tTable);
end;

procedure TfrmMain.Table1AfterClose(DataSet: TDataSet);
begin
  UpdateIndexCombo(nil);
end;

procedure TfrmMain.ixComboChange(Sender: TObject);
begin
  with Sender as tComboBox do
    table1.IndexName := tIndexDef(Items.Objects[ItemIndex]).Name;
end;


end.

